"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var proxyquire = require("proxyquire");
var assert = require("power-assert");
var sinon = require("sinon");
var mongo_connector_1 = require("../mongo_connector");
var dbStub = sinon.stub();
var MongoStub = {
    MongoClient: {
        connect: function () { return ({
            db: dbStub,
            close: sinon.stub()
        }); }
    }
};
var MongoConnectorStub = proxyquire('../mongo_connector', {
    mongodb: MongoStub
}).MongoConnector;
describe('connector', function () {
    it('will initialise url', function () { return __awaiter(_this, void 0, void 0, function () {
        var url, connector;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    url = 'mongodb://url';
                    dbStub.reset();
                    connector = new MongoConnectorStub(url, 'clara');
                    return [4, connector.connect()];
                case 1:
                    _a.sent();
                    assert.equal(connector.url, url);
                    assert(dbStub.calledOnce);
                    return [2];
            }
        });
    }); });
    it('will not initialise db on error and logs error', function () {
        return __awaiter(this, void 0, void 0, function () {
            var url, successSpy, connector, ex_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        url = 'mongodb://url';
                        successSpy = sinon.spy();
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        connector = new mongo_connector_1.MongoConnector(url, 'clara');
                        return [4, connector.connect()];
                    case 2:
                        _a.sent();
                        assert.equal(connector.db, undefined);
                        return [3, 4];
                    case 3:
                        ex_1 = _a.sent();
                        assert.equal(ex_1.message, 'Connection Error to mongodb://url/clara');
                        return [3, 4];
                    case 4: return [2];
                }
            });
        });
    });
    it('will initialise and calls back', function () { return __awaiter(_this, void 0, void 0, function () {
        var url, db, startedSpy, connector;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    url = 'mongodb://url';
                    db = {};
                    startedSpy = sinon.spy();
                    dbStub.returns(db);
                    connector = new MongoConnectorStub(url, 'clara');
                    return [4, connector.connect(startedSpy)];
                case 1:
                    _a.sent();
                    assert.equal(connector.db, db);
                    assert(startedSpy.calledOnce);
                    return [2];
            }
        });
    }); });
    it('will create a new collection', function () { return __awaiter(_this, void 0, void 0, function () {
        var url, db, connector, collectionName, col;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    url = '';
                    db = {
                        collection: sinon.spy()
                    };
                    dbStub.returns(db);
                    connector = new MongoConnectorStub(url, 'clara');
                    return [4, connector.connect()];
                case 1:
                    _a.sent();
                    assert.equal(connector.db, db);
                    collectionName = 'Col';
                    col = connector.collection(collectionName);
                    assert(db.collection.calledWith(collectionName));
                    return [2];
            }
        });
    }); });
    it('can dispose database', function () { return __awaiter(_this, void 0, void 0, function () {
        var connector, db;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    connector = new MongoConnectorStub(null, 'clara');
                    db = {
                        dropDatabase: sinon.stub()
                    };
                    dbStub.returns(db);
                    return [4, connector.connect()];
                case 1:
                    _a.sent();
                    return [4, connector.dispose()];
                case 2:
                    _a.sent();
                    sinon.assert.calledOnce(connector.db.dropDatabase);
                    sinon.assert.calledOnce(connector.client.close);
                    return [2];
            }
        });
    }); });
});
